/*
 * Copyright (C) 2004 Dell Inc.
 * Chieng-Hwa Lin <chieng-hwa_lin@dell.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __GPIOIOCTS_H_INCLUDED__
#define __GPIOIOCTS_H_INCLUDED__


#ifdef __cplusplus
extern "C" {
#endif


#pragma pack(1)


//---------------------------------------------------------------------
// Equates used for ioctl request status fields
//---------------------------------------------------------------------
#define IOCTL_STATUS_UNSUCCESSFUL               (-1)
#define IOCTL_STATUS_SUCCESS                    (0)
#define IOCTL_STATUS_CMD_NOT_IMPLEMENTED        (1)
#define IOCTL_STATUS_DEAD_CARD                  (2)
#define IOCTL_STATUS_CMD_BAD                    (3)
#define IOCTL_STATUS_LOCK_FAILED                (4)
#define IOCTL_STATUS_BAD_BUFFER                 (5)
#define IOCTL_STATUS_FLAKY_CARD                 (6)
#define IOCTL_STATUS_BAD_ALIGNMENT              (7)
#define IOCTL_STATUS_BAD_BUFTYPE                (8)
#define IOCTL_STATUS_CMD_PENDING				(9)
#define IOCTL_STATUS_CMD_TIMEOUT				(10)

#define MIN(x,y)                           ((x < y) ? x : y)
//---------------------------------------------------------------------
// Ioctl Command definitions
//---------------------------------------------------------------------
#define GPIO_IOC_MAGIC '2'

#define IOCTL_GPIO_WRITE_CMD _IOW(GPIO_IOC_MAGIC,  0, int)

#define IOCTL_GPIO_READ_CMD _IOW(GPIO_IOC_MAGIC,  1, int)
#define IOCTL_WAIT_FOR_FAN_INTR _IOW(GPIO_IOC_MAGIC,  2, int)
#define IOCTL_WAIT_FOR_TEMP_INTR _IOW(GPIO_IOC_MAGIC,  3, int)
#define IOCTL_WAIT_FOR_SHIFTG_INTR _IOW(GPIO_IOC_MAGIC,  4, int)
#define IOCTL_WAIT_FOR_BLADE_INTR _IOW(GPIO_IOC_MAGIC,  5, int)
#define IOCTL_WAIT_FOR_IOM_INTR _IOW(GPIO_IOC_MAGIC,  6, int)
#define IOCTL_WAIT_FOR_PSU_INTR _IOW(GPIO_IOC_MAGIC,  7, int)
#define IOCTL_WAIT_FOR_CP_INTR _IOW(GPIO_IOC_MAGIC,  8, int)
#define IOCTL_WAIT_FOR_KVM_INTR _IOW(GPIO_IOC_MAGIC,  9, int)
#define IOCTL_SIMULATE_GENERATE_BLADE_INTR _IOW(GPIO_IOC_MAGIC,  10, int)
#define IOCTL_SIMULATE_GENERATE_FAN_INTR _IOW(GPIO_IOC_MAGIC,  11, int)
#define IOCTL_SIMULATE_GENERATE_PSU_INTR _IOW(GPIO_IOC_MAGIC,  12, int)
#define IOCTL_SIMULATE_GENERATE_KVM_INTR _IOW(GPIO_IOC_MAGIC,  13, int)
#define IOCTL_SIMULATE_GENERATE_CP_INTR _IOW(GPIO_IOC_MAGIC,  14, int)
#define IOCTL_SIMULATE_GENERATE_IOM_INTR _IOW(GPIO_IOC_MAGIC,  15, int)
#define IOCTL_SIMULATE_GENERATE_SHIFTY_INTR _IOW(GPIO_IOC_MAGIC,  16, int)
#define IOCTL_GPIO_WRITE_CMD_32 _IOW(GPIO_IOC_MAGIC,  17, int)

#define IOCTL_GPIO_READ_CMD_32 _IOW(GPIO_IOC_MAGIC,  18, int)
#define IOCTL_ENABLE_KVM_INTR _IOW(GPIO_IOC_MAGIC,  19, int)
#define IOCTL_ENABLE_PSU_INTR _IOW(GPIO_IOC_MAGIC,  20, int)
#define IOCTL_ENABLE_IOM_INTR _IOW(GPIO_IOC_MAGIC,  21, int)
#define IOCTL_ENABLE_FAN_INTR _IOW(GPIO_IOC_MAGIC,  22, int)
#define IOCTL_ENABLE_SHIFTY_INTR _IOW(GPIO_IOC_MAGIC,  23, int)
#define IOCTL_ENABLE_BLADE_INTR _IOW(GPIO_IOC_MAGIC,  24, int)
#define IOCTL_ENABLE_CP_INTR _IOW(GPIO_IOC_MAGIC,  25, int)
#define IOCTL_DISABLE_KVM_INTR _IOW(GPIO_IOC_MAGIC,  26, int)
#define IOCTL_DISABLE_PSU_INTR _IOW(GPIO_IOC_MAGIC,  27, int)
#define IOCTL_DISABLE_IOM_INTR _IOW(GPIO_IOC_MAGIC,  28, int)
#define IOCTL_DISABLE_FAN_INTR _IOW(GPIO_IOC_MAGIC,  29, int)
#define IOCTL_DISABLE_SHIFTY_INTR _IOW(GPIO_IOC_MAGIC,  30, int)
#define IOCTL_DISABLE_BLADE_INTR _IOW(GPIO_IOC_MAGIC,  31, int)
#define IOCTL_DISABLE_CP_INTR _IOW(GPIO_IOC_MAGIC,  32, int)
#define IOCTL_WAIT_FOR_ECC_INTR _IOW(GPIO_IOC_MAGIC,  33, int)
#define IOCTL_SIMULATE_GENERATE_ECC_INTR _IOW(GPIO_IOC_MAGIC,  34, int)
#define IOCTL_RESET_PPC_I2C1 _IOW(GPIO_IOC_MAGIC,  35, int)
#define IOCTL_RESET_XILINX_IOM1 _IOW(GPIO_IOC_MAGIC,  36, int)
#define IOCTL_RESET_XILINX_IOM2 _IOW(GPIO_IOC_MAGIC,  37, int)
#define IOCTL_RESET_XILINX_PSU1 _IOW(GPIO_IOC_MAGIC,  38, int)
#define IOCTL_RESET_XILINX_PSU2 _IOW(GPIO_IOC_MAGIC,  39, int)
#define IOCTL_RESET_XILINX_KVM _IOW(GPIO_IOC_MAGIC,  40, int)
#define IOCTL_RESET_PPC_I2C_0 _IOW(GPIO_IOC_MAGIC,  41, int)
#define IOCTL_RESET_PPC_I2C_1 _IOW(GPIO_IOC_MAGIC,  42, int)
#define IOCTL_INIT_XILINX_KVM _IOW(GPIO_IOC_MAGIC,  43, int)
#define IOCTL_INIT_XILINX_PSU1 _IOW(GPIO_IOC_MAGIC,  44, int)
#define IOCTL_INIT_XILINX_PSU2 _IOW(GPIO_IOC_MAGIC,  45, int)
#define IOCTL_INIT_XILINX_IOM1 _IOW(GPIO_IOC_MAGIC,  46, int)
#define IOCTL_INIT_XILINX_IOM2 _IOW(GPIO_IOC_MAGIC,  47, int)
#define IOCTL_READ_SMBUS_TIMEOUT_STAT _IOW(GPIO_IOC_MAGIC,  48, int)
#define IOCTL_GPIO_SETBITS _IOW(GPIO_IOC_MAGIC, 49, int)
#define IOCTL_GPIO_SETBITS32 _IOW(GPIO_IOC_MAGIC, 50, int)
#define IOCTL_GPIO_CLRBITS _IOW(GPIO_IOC_MAGIC, 51, int)
#define IOCTL_GPIO_CLRBITS32 _IOW(GPIO_IOC_MAGIC, 52, int)
#define IOCTL_TESTING_CORE_DUMP _IOW(GPIO_IOC_MAGIC, 53, int)
#define IOCTL_GET_BLADE_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 54, int)
#define IOCTL_GET_FAN_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 55, int)
#define IOCTL_GET_IOM_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 56, int)
#define IOCTL_GET_PSU_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 57, int)
#define IOCTL_GET_CP_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 58, int)
#define IOCTL_GET_KVM_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 59, int)
#define IOCTL_GET_SHIFTY_INTR_QUEUE _IOR(GPIO_IOC_MAGIC, 60, int)
#define IOCTL_DISABLE_XILINX_IOM1  _IOW(GPIO_IOC_MAGIC, 61, int)
#define IOCTL_DISABLE_XILINX_IOM2  _IOW(GPIO_IOC_MAGIC, 62, int)

#pragma pack()


#ifdef __cplusplus
}
#endif


#endif //  __GPIOIOCTS_H_INCLUDED__ 



