/*
 * Copyright (C) 2004 Dell Inc.
 * Chieng-Hwa Lin <chieng-hwa_lin@dell.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __GPIOTYPES_H_INCLUDED__
#define __GPIOTYPES_H_INCLUDED__


#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(1)

#define STATUS_SUCCESS                          0
#define STATUS_UNSUCCESSFUL              0x00000001
#define STATUS_NO_SUCH_DEVICE            0x0000000e
#define STATUS_INSUFFICIENT_RESOURCES    0xC000009A     // ntsubauth 

#define SRAM2_BASE			0xB1700000

/**********************/
/* Memory Definitions */
/**********************/
#define AMD_MEMORY_MAP_OFFSET    SRAM2_BASE
#define AMD_ATA_DEVICE_A			(AMD_MEMORY_MAP_OFFSET+0x0200)
#define AMD_ATA_DEVICE_B			(AMD_MEMORY_MAP_OFFSET+0x0300)
#define AMD_ATA_BUFFER_MEMORY_0	(AMD_MEMORY_MAP_OFFSET+0x1000)
#define AMD_ATA_BUFFER_MEMORY_1  (AMD_MEMORY_MAP_OFFSET+0x1800)
#define AMD_ESM_INT_STAT_REG     (AMD_MEMORY_MAP_OFFSET+0x002A)
#define AMD_ESM_INT_MASK_REG     (AMD_MEMORY_MAP_OFFSET+0x002C)
#define AMD_ESM_INT_EVENT_REG    (AMD_MEMORY_MAP_OFFSET+0x002F)
#define FPGA_REV_REG				(AMD_MEMORY_MAP_OFFSET+0x28)

/* extended GPIO config registers */
#define AMD_EGPIO_00_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x00)
#define AMD_EGPIO_01_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x02)
#define AMD_EGPIO_02_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x04)
#define AMD_EGPIO_03_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x06)
#define AMD_EGPIO_04_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x08)
#define AMD_EGPIO_05_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x0A)
#define AMD_EGPIO_06_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x0C)
#define AMD_EGPIO_07_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x0E)
#define AMD_EGPIO_08_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x10)
#define AMD_EGPIO_09_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x12)
#define AMD_EGPIO_10_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x14)
#define AMD_EGPIO_11_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x16)
#define AMD_EGPIO_12_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x18)
#define AMD_EGPIO_13_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x1A)
#define AMD_EGPIO_14_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x1C)
#define AMD_EGPIO_15_CONFIG       (AMD_MEMORY_MAP_OFFSET+0x1E)
#define AMD_EGPIO_INT_STAT        (AMD_MEMORY_MAP_OFFSET+0x20)
#define AMD_EGPIO_INT_MASK        (AMD_MEMORY_MAP_OFFSET+0x22)
#define AMD_EGPIO_OUTPUT          (AMD_MEMORY_MAP_OFFSET+0x24)
#define AMD_EGPIO_INPUT           (AMD_MEMORY_MAP_OFFSET+0x26)

#define HAL_AMD_EGPIO_INT_GET_STATUS         ( *(volatile u16*)AMD_EGPIO_INT_STAT )
#define HAL_AMD_EGPIO_INT_CLEAR_STATUS(x)    { *(volatile u16*)AMD_EGPIO_INT_STAT = (x); }
#define HAL_AMD_EGPIO_INT_GET_MASK           ( *(volatile u16*)AMD_EGPIO_INT_MASK )
#define HAL_AMD_EGPIO_INT_SET_MASK(x)        { *(volatile u16*)AMD_EGPIO_INT_MASK = (x); }
#define HAL_AMD_EGPIO_OUTPUT(x)              { *(volatile u16*)AMD_EGPIO_OUTPUT = (x); }
#define HAL_AMD_EGPIO_INPUT                  ( *(volatile u16*)AMD_EGPIO_INPUT )
#define HAL_AMD_EGPIO_CONFIG(reg,x)          { *(volatile u16*)reg = (x); }

/*	Set/Clear a XGPIO data register bit */
#define HAL_AMD_EGPIO_SET_OUTPUT_BIT(x)   {HAL_AMD_EGPIO_OUTPUT(HAL_AMD_EGPIO_INPUT | (x));}
#define HAL_AMD_EGPIO_CLEAR_OUTPUT_BIT(x) {HAL_AMD_EGPIO_OUTPUT(HAL_AMD_EGPIO_INPUT & ~(x));}


/***************************/
/* Generic Bit Definitions */
/***************************/
#define BIT_0		 0x0001
#define BIT_1		 0x0002
#define BIT_2		 0x0004
#define BIT_3		 0x0008
#define BIT_4		 0x0010
#define BIT_5		 0x0020
#define BIT_6		 0x0040
#define BIT_7		 0x0080
#define BIT_8		 0x0100
#define BIT_9		 0x0200
#define BIT_10		 0x0400
#define BIT_11		 0x0800
#define BIT_12		 0x1000
#define BIT_13		 0x2000
#define BIT_14		 0x4000
#define BIT_15		 0x8000
#define HIGH_BYTE	 0xff00
#define LOW_BYTE	 0x00ff
#define TWELVE_BITS  0x0fff

/* extended GPIO Config reginster bit definitions */
#define EGPIO_CONFIG_DIR           BIT_0
#define EGPIO_CONFIG_IEN           BIT_1
#define EGPIO_CONFIG_ITYPE0        BIT_2
#define EGPIO_CONFIG_ITYPE1        BIT_3
#define EGPIO_CONFIG_TOGL          BIT_8
#define EGPIO_CONFIG_RATE0         BIT_9
#define EGPIO_CONFIG_RATE1         BIT_10
#define EGPIO_CONFIG_RATE2         BIT_11
#define EGPIO_CONFIG_RATE3         BIT_12
#define EGPIO_CONFIG_TOGN          BIT_13

/*
 * /proc file system
 */
#define CMC_FPGA_BASE_ADDRESS		0xa0000000	// FPGA GPIO base address
#define CMC_FPGA_MODE_0_REG_OFFSET	0x0008		// FPGA mode 0 register (lower 16-bits)
#define MODE_VENDOR_SOFT		0x0002		/*set by racadm util cmd*/
#define VENDOR_MODE	"vendor_mode"
#define SOFT_VENDOR_MODE	"racadm_vendor_mode"
#define PASSWORD_MODE	"password_mode"
extern void create_cmc_file(char *);
extern void remove_cmc_file(char *);

#pragma pack()

#ifdef __cplusplus
}
#endif
#endif  // __GPIOTYPES_H_INCLUDED__ 


