/////////////////////////////////////////////////////////////////////////
//
//          DELL COMPUTER CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Computer Corporation and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 2011 Dell Computer Corporation. All Rights Reserved.
//
// Module Name: gthifi.c
//
/////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include "cmc_gpio.h"
#include "cmc_gpio_prototypes.h"

#define MAX_HIFI_SLEDS  CMC_MAX_SWDH_BLADES
#define USER_X (-1)

static int get_num_or_x( void )
{
#define TMPBUFSIZ 80

    char tmp[TMPBUFSIZ];

    fgets(tmp, sizeof(tmp), stdin);
    if ('x' == tmp[0] || 'X' == tmp[0]) {
        return(USER_X);
    }

    return(atoi(tmp));
}

static void control_qblade_menu( int blade_slot, hifi_qblade_id_t qblade_id )
{
    char tmp[ TMPBUFSIZ ];
    int rc;
    int auxpwr_on = 0;
    qblades_info_t qblades_info;

    int done = 0;
    while ( !done ) {
        rc = CMCHiFiGetQbladesInfo( blade_slot, &qblades_info );
        if ( 0 == rc ) {
            printf( "(raw qbi = %08X)\n", qblades_info.qbi_all);
            auxpwr_on = 0;
            switch ( qblade_id ) {
                case HIFI_QBLADE_A:
                    if ( qblades_info.qbi_bits.A_auxpwr_on ) {
                        auxpwr_on = 1;
                    }
                    break;
                case HIFI_QBLADE_B:
                    if ( qblades_info.qbi_bits.B_auxpwr_on ) {
                        auxpwr_on = 1;
                    }
                    break;
                case HIFI_QBLADE_C:
                    if ( qblades_info.qbi_bits.C_auxpwr_on ) {
                        auxpwr_on = 1;
                    }
                    break;
                case HIFI_QBLADE_D:
                    if ( qblades_info.qbi_bits.D_auxpwr_on ) {
                        auxpwr_on = 1;
                    }
                    break;
            }
            if ( auxpwr_on ) {
                printf( "AUX power is ON\n" );
                printf( "\tp - turn off AUX power to iDRAC\n" );
                printf( "\tx - exit\n" );
                printf( "Select choice: " );
                fgets( tmp, sizeof( tmp ), stdin );
                switch ( tmp[ 0 ] ) {
                    case 'p': {
                        rc = CMCHiFiQbladeAuxPowerControl( blade_slot, qblade_id, CMC_TURN_PS_OFF );
                        if ( 0 != rc ) {
                            printf( "failed to turn off aux power to qblade, rc = %d\n", rc );
                        }
                        break;
                    }
                    case 'x':
                        done = 1;
                }
            }
            else {
                printf( "AUX power is OFF\n" );
                printf( "\tp - turn on AUX power to iDRAC\n" );
                printf( "\tx - exit\n" );
                printf( "Select choice: " );
                fgets( tmp, sizeof( tmp ), stdin );
                switch ( tmp[ 0 ] ) {
                    case 'p': {
                        rc = CMCHiFiQbladeAuxPowerControl( blade_slot, qblade_id, CMC_TURN_PS_ON );
                        if ( 0 != rc ) {
                            printf( "failed to turn on aux power to qblade, rc = %d\n", rc );
                        }
                        break;
                    }
                    case 'x':
                        done = 1;
                }
            }
        }
        else {
            printf( "failed to retrieve qblade info, rc = %d\n", rc );
            done = 1;
        }
    }
}

static void control_qblades_menu( int blade_slot, qblades_info_t  * qblades_info )
{
    char tmp[ TMPBUFSIZ ];

    int done = 0;
    while ( !done ) {
        printf("The following qblades are present: ");
        if ( qblades_info->qbi_A_present ) {
            printf( "A " );
        }
        if ( qblades_info->qbi_B_present ) {
            printf( "B " );
        }
        if ( qblades_info->qbi_C_present ) {
            printf( "C " );
        }
        if ( qblades_info->qbi_D_present ) {
            printf( "D " );
        }
        printf( "\n" );
        printf("\nSelect a qblade (a, b, c, d), or \"x\" to exit: ");
        fgets( tmp, sizeof( tmp ), stdin );
        printf("user entered %c\n", tmp[ 0 ]);
        switch ( tmp[ 0 ] ) {
            case 'a':
                control_qblade_menu( blade_slot, HIFI_QBLADE_A );
                break;
            case 'b':
                control_qblade_menu( blade_slot, HIFI_QBLADE_B );
                break;
            case 'c':
                control_qblade_menu( blade_slot, HIFI_QBLADE_C );
                break;
            case 'd':
                control_qblade_menu( blade_slot, HIFI_QBLADE_D );
                break;
            case 'x':
                done = 1;
                break;
        }
    }
}

static void blade_menu( int blade_slot )
{
#define TMPBUFSIZ 80

    qblades_info_t  qblades_info;
    char tmp[ TMPBUFSIZ ];

    int done = 0;
    while ( !done ) {
        // Read the AUX power supply status for all blades
        int rc, ps_status;

        rc = CMCReadBladePSOnOffStatus( &ps_status );
        if (0 != rc) {
            printf( "failed to read blade power supply status, rc = %d\n", rc );
            break;
        }
        if ( ps_status & ( 1 << (blade_slot - 1) ) )
        {
            printf( "AUX power is ON\n" );
            int sts_val = 0, ctl_val = 0;
            rc = CMCReadBladeStatus( blade_slot, &sts_val );
            if ( 0 != rc ) {
                printf( "failed to read blade status, rc = %d\n", rc );
                break;
            }
            else {
                // Detect blade geometry
                if ( CMC_BLDSTS_IS_SWSH( sts_val ) ) {
                    printf( "blade is single-width, single-height\n" );
                }
                else if ( CMC_BLDSTS_IS_SWDH( sts_val ) ) {
                    printf( "blade is single-width, double-height\n" );
                }
                else if ( CMC_BLDSTS_IS_DWSH( sts_val ) ) {
                    printf( "blade is double-width, single-height\n" );
                }
                else if ( CMC_BLDSTS_IS_DWDH( sts_val ) ) {
                    printf( "blade is double-width, double-height\n" );
                }
                else {
                    printf( "blade is unknown geometry\n" );
                }
                // Detect blade type
                if ( CMC_BLDSTS_IS_HIFI( sts_val ) ) {
                    printf( "blade is a HiFi\n" );
                    rc = CMCReadBladeControlStatus( blade_slot, &ctl_val );
                    if ( 0 != rc ) {
                        printf( "failed to read blade control status, rc = %d\n", rc );
                        break;
                    }
                    else {
                        if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK1 ) {
                            printf( "AltDecodeACK1 is asserted\n" );
                            // Once the blade recognizes that AltDecodeACK1 has been
                            // asserted, it should switch to alternate-encoding,
                            // which will enable us to detect the presence of qblades.
                            rc = CMCHiFiGetQbladesInfo( blade_slot, &qblades_info );
                            if ( 0 == rc ) {
                                printf( "\tQblades Present: " );
                                if ( qblades_info.qbi_A_present ) {
                                    printf( "A " );
                                }
                                if ( qblades_info.qbi_B_present ) {
                                    printf( "B " );
                                }
                                if ( qblades_info.qbi_C_present ) {
                                    printf( "C " );
                                }
                                if ( qblades_info.qbi_D_present ) {
                                    printf( "D " );
                                }
                                printf( "\n" );
                            }
                            else {
                                printf( "failed to detect qblade presence, rc = %d\n", rc );
                            }

                            if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2 ) {
                                printf( "AltDecodeACK2 is asserted\n" );
                                if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_THROTTLE_ALL) {
                                    printf("QBlades are throttled.\n");
                                }
                                else {
                                    printf("QBlades are unthrottled.\n");
                                }
                            }
                            else {
                                printf( "AltDecodeACK2 is deasserted\n" );
                            }
                        }
                        else {
                            printf( "AltDecodeACK1 is deasserted\n" );
                        }
                    }
                }
                else if ( CMC_BLDSTS_IS_COLOSSUS( sts_val ) ) {
                    printf( "blade is a Colossus\n" );
                }
                else { // Ordinary blade
                }
            }
            printf( "\tp - turn off AUX power\n" );
            if ( CMC_BLDSTS_IS_HIFI( sts_val ) ) {
                if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK1 ) {
                    if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2 ) {
                        printf( "\tt - throttle all blades.\n");
                        printf( "\tu - unthrottle all blades.\n");
                        printf( "\tc - control qblades\n" );
                    }
                    else {
                        printf( "\t2 - assert AltDecodeACK2\n" );
                    }
                }
                else {
                    printf( "\t1 - assert AltDecodeACK1\n" );
                }
            }
            printf( "\tx - exit\n" );
            printf( "Select choice: " );
            fgets( tmp, sizeof( tmp ), stdin );
            switch ( tmp[ 0 ] ) {
                case '1': // Assert AltDecodeACK1
                    if ( CMC_BLDSTS_IS_HIFI( sts_val ) ) {
                        if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK1 ) {
                        }
                        else {
                            // Assert AltDecodeACK1
                            rc = CMCHiFiAltDecodeACK1( blade_slot );
                            if ( 0 != rc ) {
                                printf( "failed to assert AltDecodeACK1, rc = %d\n", rc );
                            }
                        }
                    }
                    break;
                case '2': // Assert AltDecodeACK2
                    if ( CMC_BLDSTS_IS_HIFI( sts_val ) ) {
                        if ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2 ) {
                        }
                        else {
                            // Assert AltDecodeACK2
                            int on_qblades_mask = 0;
                            rc = CMCHiFiAltDecodeACK2( blade_slot, on_qblades_mask );
                            if ( 0 != rc ) {
                                printf( "failed to assert AltDecodeACK2, rc = %d\n", rc );
                            }
                        }
                    }
                    break;
                case 'p': { // Turn off AUX power to the sled
                    if ( CMC_BLDSTS_IS_HIFI( sts_val ) ) {
                        rc = CMCHiFiSledAuxPowerControl( blade_slot, CMC_TURN_PS_OFF );
                    } else {
                        int mask = (1 << (blade_slot - 1)) | (1 << ((blade_slot - 1) + CMC_MAX_SWDH_BLADES));
                        rc = CMCTurnBladePSOnOff( mask, CMC_TURN_PS_OFF );
                    }
                    if ( 0 != rc ) {
                        printf( "failed to turn off aux power to sled, rc = %d\n", rc );
                    }
                    break;
                }
                case 'c': // Control qblade iDRAC, server
                    if ( CMC_BLDSTS_IS_HIFI( sts_val ) &&
                         ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK1 ) &&
                         ( ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2 ) ) {
                        control_qblades_menu( blade_slot, &qblades_info );
                    }
                    break;
                case 'u':
                    if ( (ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2) && CMC_BLDSTS_IS_HIFI( sts_val )){
                        if (  ! (rc == CMCQbladesSetThrottleAll( blade_slot, CMC_UNTHROTTLE)))
                            printf("  Failed to unthrottle.\n\n\n");
                    }
                	break;
                case 't':
                    if ( (ctl_val & CMC_BLDCTL_HIFI_MSTR_ACK2) && CMC_BLDSTS_IS_HIFI( sts_val )){
                        if (  ! (rc == CMCQbladesSetThrottleAll( blade_slot, CMC_THROTTLE)))
                            printf("  Failed to throttle.\n\n\n");
                    }
                	break;
                case 'x': // Exit
                    done = 1;
            }
        }
        else
        {
            printf( "AUX power is OFF\n" );
            printf( "\tp - turn on AUX power to this slot\n" );
            if (blade_slot <= CMC_MAX_SWDH_BLADES) {
                printf( "\tP - turn on AUX power to this slot and to lower slot\n" );
            }
            printf( "\tx - exit\n" );
            printf( "Select choice: " );
            fgets( tmp, sizeof( tmp ), stdin );
            switch ( tmp[ 0 ] ) {
                case 'p': {
                    int mask = (1 << (blade_slot - 1));
                    rc = CMCTurnBladePSOnOff( mask, CMC_TURN_PS_ON );
                    if ( 0 != rc ) {
                        printf( "failed to turn on aux power to slot %d, rc = %d\n", blade_slot, rc );
                    }
                    break;
                }
                case 'P':
                    // Use HiFi function just for convenience of testing.
                    rc = CMCHiFiSledAuxPowerControl( blade_slot, CMC_TURN_PS_ON );
                    if ( 0 != rc ) {
                        printf( "failed to turn on aux power to sled, rc = %d\n", rc );
                    }
                    break;
                case 'x':
                    done = 1;
            }
        }
    }
}

static void blades_menu( int *blades )
{
    int slot;

    while ( 1 ) {
        printf("The following slots contain a blade: ");
        for (slot = 1; slot <= CMC_MAX_BLADE_SLOTS; slot++) {
            if (blades[slot]) {
                printf("%d ", slot);
            }
        }
        printf("\nSelect a slot number, or \"x\" to exit: ");
        slot = get_num_or_x();
        if (USER_X == slot) {
            return;
        }
        else {
            printf("user entered slot %d\n", slot);
            blade_menu( slot );
        }
    }
}

static int find_blades( int *blades )
{
    int slot, rc, nblades, bldsts;

    for (slot = 1, nblades = 0; slot <= CMC_MAX_BLADE_SLOTS; slot++) {
        rc = CMCReadBladeStatus(slot, &bldsts);
        if (0 == rc && ((bldsts & CMC_BLDSTS_BLADE_PSNT) || (bldsts & CMC_BLDSTS_BLNK_PSNT))) {
            blades[slot] = 1;
            nblades++;
        } else {
            blades[slot] = 0;
        }
    }

    return(nblades);
}

void HiFiMenu( void )
{
    int nblades, blades[CMC_MAX_BLADE_SLOTS + 1]; // 1-based array of slot numbers

    printf("CMC 4.2 HiFi Support Menu\n\n");

    nblades = find_blades( blades );
    if (0 == nblades) {
        printf("No blades were found.\n");
    } else {
        blades_menu( blades );
    }
}
