/* 
    CMC RPC Client library
    Copyright (C) 2010 Dell, Inc

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
#include <stdlib.h>
#include <stdarg.h>
#include <curl/curl.h>
#include <jansson.h>
#include "cmcrpc-c.h"

int 
CMCGetLocalCMCState_rpc()
{
    json_t *request = json_object();
    json_t *j_resp;
    json_t *j_result;
    json_t *j_returncode;
    char *json;
    CURLcode cret;
    cmcrpc_t *rpc;
    int rc = 0;
    json_error_t error;

    /* setup request */
    json_object_set_new(request, "method", json_string("CMCGetLocalCMCState"));
    json_object_set_new(request, "id", json_null());
    json_object_set_new(request, "params", json_null());

    /* convert request object into a json formatted string */
    json = json_dumps(request, 0);

    curl_global_init(CURL_GLOBAL_NOTHING);


    rpc = cmcrpc_new(0, json);
    if ((cret = cmcrpc_run(rpc)) != CURLE_OK) {
	goto done;
    }
    /* Handle response */
    j_resp = json_loads(cmcrpc_data(rpc),  &error);
    if (!j_resp) {
	fprintf(stderr, "%d\n%s\n", error.line, error.text);
        goto done;
    }

    /* pull out the json-rpc result */
    j_result = json_object_get(j_resp, "result");
    j_returncode = json_object_get(j_result, "returncode");
    rc = json_integer_value(j_returncode);

    json_decref(j_result);
    json_decref(j_returncode);
    json_decref(j_resp);

done:
    
    json_decref(request);
    free(json);
    cmcrpc_free(rpc);
    curl_global_cleanup();

    return rc;
}

